Package: frma
Version: 1.63.0
Date: 2017-11-08
Title: Frozen RMA and Barcode
Description: Preprocessing and analysis for single microarrays and microarray batches.
Author: Matthew N. McCall <mccallm@gmail.com>, Rafael A. Irizarry <rafa@jhu.edu>, with contributions from Terry Therneau
Maintainer: Matthew N. McCall <mccallm@gmail.com>
Depends: R (>= 2.10.0), Biobase (>= 2.6.0)
Imports: Biobase, MASS, DBI, affy, methods, oligo, oligoClasses,
        preprocessCore, utils, BiocGenerics
Suggests: hgu133afrmavecs, frmaExampleData
biocViews: Software, Microarray, Preprocessing
License: GPL (>= 2)
URL: http://bioconductor.org
git_url: https://git.bioconductor.org/packages/frma
git_branch: devel
git_last_commit: 06e0f3f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-02
NeedsCompilation: no
Packaged: 2025-11-02 22:40:52 UTC; biocbuild

importFrom(Biobase, phenoData, experimentData)
importClassesFrom(Biobase, AnnotatedDataFrame, AssayData, eSet,
                  ExpressionSet, MIAME, Versioned, VersionedBiobase,
                  Versions)
importClassesFrom(methods, character, matrix)
importMethodsFrom(methods, initialize)
importMethodsFrom(Biobase, annotatedDataFrameFrom, exprs, se.exprs, "se.exprs<-", annotation, sampleNames)
importMethodsFrom(affy, cdfName, pm, "pm<-", pmindex, probeNames)
importMethodsFrom(oligo, backgroundCorrect)
importMethodsFrom(oligoClasses, db)
importMethodsFrom(DBI, dbGetQuery)

importFrom(Biobase, assayDataElement, assayDataElementReplace, assayDataNew)
importFrom(MASS, rlm)
importFrom(affy, bg.correct.rma, cleancdfname, ReadAffy, plotDensity)
importFrom(preprocessCore, normalize.quantiles.use.target, rcModelPLM,
           rcModelWPLM, subColSummarizeAvg, subColSummarizeMedian)
importFrom(utils, data, citation, toBibtex)
importFrom(methods, is, callNextMethod, new, setAs, as)
importFrom(BiocGenerics, weights, residuals)

export(frma, barcode, GNUSE)
exportClasses(frmaExpressionSet)
exportMethods(initialize, se.exprs, weights, residuals, randomeffects, 
              "se.exprs<-", "weights<-", "residuals<-", "randomeffects<-")

frmaFeatureSet <- function(object, background, normalize, summarize, target, input.vecs, output.param, verbose){

  if(background == "rma"){
    if(verbose) message("Background Correcting ...\n")
    object <- backgroundCorrect(object, verbose=verbose)
  }
  
  featureInfo <- getFidProbeset(object)
  pmi <- featureInfo[["fid"]]
  pns <- as.character(featureInfo[["fsetid"]])
  pms <- exprs(object)[pmi,, drop=FALSE]
  
  if(normalize == "quantile"){
    if(verbose) message("Normalizing ...\n")
    pms <- normalize.quantiles.use.target(pms, input.vecs$normVec)
  }
  
  ## first summarize at the probeset-level
  if(verbose) message("Summarizing...\n")
  pms <- log2(pms)
  
  if(summarize == "average"){  
    exprs <- subColSummarizeAvg((pms-input.vecs$probeVec), pns)
    weights <- NULL
    stderr <- NULL
  }
  
  if(summarize %in% c("median","median_polish")){
      if(summarize == "median_polish") message("Note: median and median_polish summarization methods are identical in fRMA.\n")
      exprs <- subColSummarizeMedian((pms-input.vecs$probeVec), pns)
      weights <- NULL
      stderr <- NULL
  }
  
  if(summarize == "weighted_average"){  
    w <- 1/(input.vecs$probeVarWithin + input.vecs$probeVarBetween)
    if(any(input.vecs$probeVarWithin==0) | any(input.vecs$probeVarBetween==0)) message("Either probeVarWithin or probeVarBetween is 0 for some probes -- setting corresponding weights to 1")
    w[w==Inf] <- 1
    exprs <- subColSummarizeAvg((pms-input.vecs$probeVec)*w, pns)
    W <- as.vector(rowsum(w, pns, reorder=FALSE))
    exprs <- (exprs/W)*as.vector(rowsum(rep(1,length(pns)),pns,reorder=FALSE))
    weights <- NULL
    stderr <- NULL
  }
  
  if(summarize == "robust_weighted_average"){
    w <- 1/(input.vecs$probeVarWithin + input.vecs$probeVarBetween)
    if(any(input.vecs$probeVarWithin==0) | any(input.vecs$probeVarBetween==0)) message("Either probeVarWithin or probeVarBetween is 0 for some probes -- setting corresponding weights to 1")
    w[w==Inf] <- 1
    N <- 1:dim(pms)[1]
    S <- split(N, pns)
    fit <- lapply(1:length(S), function(i) {
      s <- S[[i]]
      rwaFit2(pms[s,, drop=FALSE], w[s], input.vecs$probeVec[s], input.vecs$probesetSD[i]) 
    })
    exprs <- matrix(unlist(lapply(fit, function(x) x$Estimates)), ncol=ncol(pms), byrow=TRUE)
    rownames(exprs) <- names(S)
    colnames(exprs) <- sampleNames(object)
    if("weights" %in% output.param){
      weights <- matrix(unlist(lapply(fit, function(x) x$Weights)), ncol=ncol(pms), byrow=TRUE)
      rownames(weights) <- pmi
      colnames(weights) <- sampleNames(object)
    } else weights <- NULL
    stderr <- matrix(unlist(lapply(fit, function(x) x$StdErrors)), ncol=ncol(pms), byrow=TRUE)
    rownames(stderr) <- names(S)
    colnames(stderr) <- sampleNames(object)
  }
  
  ## if selected, use the probeset-level summaries to summarize to another level
  if(target!="probeset"){    
    if(target=="core"){
      featureInfo <- getFidMetaProbesetCore(object)
      exonvec <- input.vecs$probeVecCore
      map <- input.vecs$mapCore
    }
    if(target=="full"){
      featureInfo <- getFidMetaProbesetFull(object)
      exonvec <- input.vecs$probeVecFull
      map <- input.vecs$mapFull
    }
    if(target=="extended"){
      featureInfo <- getFidMetaProbesetExtended(object)
      exonvec <- input.vecs$probeVecExt
      map <- input.vecs$mapExt
    }
    exprs.tmp <- exprs[match(map[,2],rownames(exprs)),,drop=FALSE]
    pns <- map[,1]
    
    if(summarize == "average" | summarize == "weighted_average"){  
      exprs <- subColSummarizeAvg(exprs.tmp - exonvec, pns)
      weights <- NULL
      stderr <- NULL
    }
    
    if(summarize %in% c("median","median_polish")){
        exprs <- subColSummarizeMedian(exprs.tmp - exonvec, pns)
        weights <- NULL
        stderr <- NULL
    }
    
    if(summarize == "robust_weighted_average"){
      N <- 1:dim(exprs.tmp)[1]
      S <- split(N, pns)
      fit <- lapply(1:length(S), function(i) {
        s <- S[[i]]
        evec.tmp <- exonvec[s]
        evec.tmp[1] <- evec.tmp[1]-sum(evec.tmp)
        rcModelPLM(exprs.tmp[s,, drop=FALSE], evec.tmp)
      })
      names(fit) <- unique(pns)
      exprs <- matrix(unlist(lapply(fit, function(x) x$Estimates)), ncol=ncol(exprs.tmp), byrow=TRUE)
      rownames(exprs) <- names(fit)
      colnames(exprs) <- colnames(exprs.tmp)
      if("weights" %in% output.param){
        weights <- matrix(unlist(lapply(fit, function(x) x$Weights)), ncol=ncol(exprs.tmp), byrow=TRUE)
        rownames(weights) <- pns
        colnames(weights) <- sampleNames(object)
      } else weights <- NULL
      stderr <- matrix(unlist(lapply(fit, function(x) x$StdErrors)), ncol=ncol(exprs.tmp), byrow=TRUE)
      rownames(stderr) <- names(fit)
      colnames(stderr) <- sampleNames(object)
    }
    if("residuals" %in% output.param){
      residuals <- apply(exprs,2,function(x) rep(x, table(pns)))
      residuals <- (exprs.tmp - exonvec) - residuals
      rownames(residuals) <- pns
      colnames(residuals) <- sampleNames(object)
    } else residuals <- NULL
  } else {
    if("residuals" %in% output.param){
      residuals <- apply(exprs,2,function(x) rep(x, table(pns)))
      residuals <- (pms-input.vecs$probeVec) - residuals
      rownames(residuals) <- pns
      colnames(residuals) <- sampleNames(object)
    } else residuals <- NULL
  }
  
  colnames(exprs) <- sampleNames(object)
  
  return(list(exprs=exprs, stderr=stderr, weights=weights, residuals=residuals, gammas=NULL))
}

setMethod("se.exprs", signature(object="ExpressionSet"),
          function(object) assayDataElement(object, "se.exprs"))

setReplaceMethod("se.exprs", signature(object="ExpressionSet"), function(object, value){
  object <- assayDataElementReplace(object, "se.exprs", value)
  validObject(object)
  object
})

setClass("frmaExpressionSet",
         representation(
           se.exprs="matrix",             
           weights="matrix",
           residuals="matrix",
           randomeffects="matrix"),
         prototype=prototype(
           new("ExpressionSet"),
           se.exprs=new("matrix"),
           weights=new("matrix"),
           residuals=new("matrix"),
           randomeffects=new("matrix")),
         contains="ExpressionSet")

batchFit <- function(y, probe, sigma, tau, maxiter=100){
  nchip <- ncol(y)
  nprobe <- nrow(y)
  y.new <- as.vector(y - probe)  #kill any names
  n <- length(y)
  wt <- matrix(rep(1/as.vector(sigma), nchip), nrow=nprobe)
  
  #rlm setup, taken from the rlm function
  irls.delta <- function(old, new)
    sqrt(sum((old - new)^2)/max(1e-20, sum(old^2)))
  psi.huber <- function(u, k=1.145)  pmin(1, k/abs(u))
  
  # Setup for linear regression
  xtx <- matrix(0., ncol=(nchip+nprobe), nrow=nchip+nprobe)
  i1 <- seq(1, length=nchip+nprobe, by=nchip+nprobe+1) #index of diagonal
  i2a <- which(row(xtx) <= nprobe & col(xtx) >nprobe)
  i2b <- c(outer(seq(nprobe, length=nprobe, by=nrow(xtx)), 1:nchip,  '+'))
  i3 <- as.vector(row(y))          #gamma coefs
  i4 <- as.vector(col(y)) + nprobe #theta coefs
  sqwt <- sqrt(wt) 
  w <- rep(1., length(y))
  
  # Initial linear regression
  xtx[i1] <- c(rowSums(wt) + 1/tau, colSums(wt))
  xtx[i2a] <- wt
  xtx[i2b] <- wt         
  xty <- c(rowSums(wt*y.new), colSums(wt*y.new))
  
  beta <- solve(xtx, xty)
  resid <- sqwt * (beta[i3] + beta[i4] - y.new)  # weighted residuals
  
  for (iter in 1:maxiter) {
    oldresid <- resid
    scale <- median(abs(resid))/.6745
    w <- matrix(psi.huber(resid/scale, k=1.345), nrow=nprobe)
    w2 <- w*wt
    xtx[i1] <- c(rowSums(w2) + 1/tau, colSums(w2))
    xtx[i2a] <- w2
    xtx[i2b] <- w2
    temp <- w2* y.new
    xty <- c(rowSums(temp), colSums(temp))
    
    beta <- solve(xtx, xty)
    resid <- sqwt * (beta[i3] + beta[i4] - y.new)  # weighted residuals
    convi <- irls.delta(oldresid, resid)
    if (convi < 1e-4) break
  }
  return(list(Estimates=beta[-(1:nprobe)], Weights=as.vector(t(w)), Gamma=beta[1:nprobe]))
}

frmaAffyBatch <- function(object, background, normalize, summarize, input.vecs, output.param, verbose){
  
  if(background == "rma"){
    if(verbose) message("Background Correcting ...\n")
    object <- bg.correct.rma(object)
  }
  
  pms <- pm(object)
  pns <- probeNames(object)
  pmi <- unlist(pmindex(object))
  
  if(!identical(as.character(pmi), names(input.vecs$normVec))){
    map <- match(as.character(pmi), names(input.vecs$normVec))
    input.vecs$normVec <- input.vecs$normVec[map]
    input.vecs$probeVec <- input.vecs$probeVec[map]
    input.vecs$probeVarWithin <- input.vecs$probeVarWithin[map]
    input.vecs$probeVarBetween <- input.vecs$probeVarBetween[map]
    if(!identical(as.character(pmi), names(input.vecs$normVec))){
      stop("Mismatch between pmindex(object) and names of input.vecs and unable to create unique mapping.")
    }
  }
  
  if(normalize == "quantile"){
    if(verbose) message("Normalizing ...\n")
    pms <- normalize.quantiles.use.target(pms, input.vecs$normVec)
  }
  
  if(verbose) message("Summarizing ...\n")
  pms <- log2(pms)
  
  if(summarize == "average"){  
    exprs <- subColSummarizeAvg((pms-input.vecs$probeVec), pns)
    weights <- NULL
    stderr <- NULL
    gammas <- NULL
  }
  
  if(summarize %in% c("median","median_polish")){
    if(summarize == "median_polish") message("Note: median and median_polish summarization methods are identical in fRMA.\n")
    exprs <- subColSummarizeMedian((pms-input.vecs$probeVec), pns)
    weights <- NULL
    stderr <- NULL
    gammas <- NULL
  }
  
  if(summarize == "weighted_average"){  
    w <- 1/(input.vecs$probeVarWithin + input.vecs$probeVarBetween)
    if(any(input.vecs$probeVarWithin==0) | any(input.vecs$probeVarBetween==0)) message("Either probeVarWithin or probeVarBetween is 0 for some probes -- setting corresponding weights to 1")
    w[w==Inf] <- 1
    exprs <- subColSummarizeAvg((pms-input.vecs$probeVec)*w, pns)
    W <- as.vector(rowsum(w, pns, reorder=FALSE))
    exprs <- (exprs/W)*as.vector(rowsum(rep(1,length(pns)),pns,reorder=FALSE))
    weights <- NULL
    stderr <- NULL
    gammas <- NULL
  }
  
  if(summarize == "robust_weighted_average"){
    w <- 1/(input.vecs$probeVarWithin + input.vecs$probeVarBetween)
    if(any(input.vecs$probeVarWithin==0) | any(input.vecs$probeVarBetween==0)) message("Either probeVarWithin or probeVarBetween is 0 for some probes -- setting corresponding weights to 1")
    w[w==Inf] <- 1
    N <- 1:dim(pms)[1]
    S <- split(N, pns)
    fit <- lapply(1:length(S), function(i) {
      s <- S[[i]]
      rwaFit2(pms[s,, drop=FALSE], w[s], input.vecs$probeVec[s], input.vecs$probesetSD[names(S)[i]]) 
    })
    
    stderr <- matrix(unlist(lapply(fit, function(x) x$StdErrors)), ncol=ncol(pms), byrow=TRUE)
    rownames(stderr) <- names(S)
    colnames(stderr) <- sampleNames(object)
    
    gammas <- NULL
  }
  
  if(summarize == "random_effect"){
    N <- 1:dim(pms)[1]
    S <- split(N, pns)
    fit <- lapply(1:length(S), function(i) {
      s <- S[[i]]
      batchFit(pms[s,], input.vecs$probeVec[s], input.vecs$probeVarWithin[s], input.vecs$probeVarBetween[s])
    })
    
    if("random_effects" %in% output.param){
      gammas <- matrix(unlist(lapply(fit, function(x) x$Gamma)), ncol=1)
      rownames(gammas) <- pmi
      colnames(gammas) <- NULL
    } else gammas <- NULL
    
    stderr <- NULL
  }
    
  if(summarize %in% c("robust_weighted_average","random_effect")){
    exprs <- matrix(unlist(lapply(fit, function(x) x$Estimates)), ncol=ncol(pms), byrow=TRUE)
    rownames(exprs) <- names(S)
    colnames(exprs) <- sampleNames(object)
    
    if("weights" %in% output.param){
      weights <- matrix(unlist(lapply(fit, function(x) x$Weights)), ncol=ncol(pms), byrow=TRUE)
      rownames(weights) <- pmi
      colnames(weights) <- sampleNames(object)
    } else weights <- NULL
  }
  
  if("residuals" %in% output.param){
    residuals <- apply(exprs, 2, function(x) rep(x, table(pns)))
    residuals <- (pms-input.vecs$probeVec) - residuals
    rownames(residuals) <- pmi
    colnames(residuals) <- sampleNames(object)
  } else residuals <- NULL
  
  return(list(exprs=exprs, stderr=stderr, weights=weights, residuals=residuals, gammas=gammas))
}

rwaFit2 <- function(x1, x2, x3, x4=NULL){
  ncols <- ncol(x1)
  w.tmp <- x2/max(x2)
  w.tmp <- matrix(rep(w.tmp, ncols), ncol=ncols)
  pe.tmp <- x3
  pe.tmp[1] <- pe.tmp[1]-sum(pe.tmp)
  rcModelWPLM(y=x1, w=w.tmp, row.effects=pe.tmp, input.scale=x4)
}

setMethod("initialize", "frmaExpressionSet",
          function(.Object,
                assayData = assayDataNew(exprs=exprs, se.exprs=se.exprs, ...),
                exprs=new("matrix"),
                se.exprs=new("matrix"),
                weights=new("matrix"),
                residuals=new("matrix"),
                randomeffects=new("matrix"),
                phenoData = annotatedDataFrameFrom  (assayData, byrow=FALSE),
                featureData = annotatedDataFrameFrom(assayData, byrow=TRUE),
                experimentData = new("MIAME"),
                annotation = new("character"), ...){
            .Object <- callNextMethod(.Object,
                                      assayData = assayData,
                                      phenoData = phenoData,
                                      experimentData = experimentData,
                                      annotation = annotation,
                                      featureData = featureData, ...)
            .Object
          })

## slot getters
setMethod("se.exprs", signature(object="frmaExpressionSet"),
          function(object) assayDataElement(object, "se.exprs"))

weights.frmaExpressionSet <- function(object, ...) object@weights
setMethod("weights", "frmaExpressionSet", weights.frmaExpressionSet)

residuals.frmaExpressionSet <- function(object, ...) object@residuals
setMethod("residuals", "frmaExpressionSet", residuals.frmaExpressionSet)

setMethod("randomeffects", signature(object="frmaExpressionSet"),
          function(object) object@randomeffects)

## validity method
setValidity("frmaExpressionSet", function(object){
  if(!is.null(se.exprs(object))){
    if(any(dim(exprs(object))!=dim(se.exprs(object)))) return("exprs and se.exprs must have the same dimensions")
  }
  if(nrow(weights(object))>0){
    if(ncol(weights(object))!=ncol(exprs(object))) return("weights must have the same number of columns as exprs")
  }
  if(nrow(residuals(object))>0){
    if(ncol(residuals(object))!=ncol(exprs(object))) return("residuals must have the same number of columns as exprs")
  }
  if(nrow(residuals(object))>0 & nrow(weights(object))>0){
    if(nrow(residuals(object)) != nrow(weights(object))) return("if not empty, residuals and weights must have the same number of rows")
  }
  if(nrow(randomeffects(object))>0 & nrow(weights(object))>0){
    if(nrow(randomeffects(object)) != nrow(weights(object))) return("if not empty, randomeffects and weights must have the same number of rows")
  }
  if(nrow(randomeffects(object))>0 & nrow(residuals(object))>0){
    if(nrow(randomeffects(object)) != nrow(residuals(object))) return("if not empty, randomeffects and residuals must have the same number of rows")
  }
  TRUE
})

## slot setters
setReplaceMethod("se.exprs", signature(object="frmaExpressionSet"), function(object, value){
  object <- assayDataElementReplace(object, "se.exprs", value)
  validObject(object)
  object
})

setReplaceMethod("weights", "frmaExpressionSet", function(object, value){
  object@weights <- value
  validObject(object)
  object
})

setReplaceMethod("residuals", "frmaExpressionSet", function(object, value){
  object@residuals <- value
  validObject(object)
  object
})

setReplaceMethod("randomeffects", "frmaExpressionSet", function(object, value){
  object@randomeffects <- value
  validObject(object)
  object
})

## coercion methods
setAs("frmaExpressionSet", "ExpressionSet", function(from){
  new("ExpressionSet", assayData=assayDataNew(exprs=exprs(from), se.exprs=se.exprs(from)), annotation=annotation(from))
})
                                            

setGeneric("randomeffects", function(object) standardGeneric("randomeffects"))
setGeneric("as.ExpressionSet", function(object) standardGeneric("as.ExpressionSet"))
setGeneric("weights<-", function(object, value) standardGeneric("weights<-"))
setGeneric("residuals<-", function(object, value) standardGeneric("residuals<-"))
setGeneric("randomeffects<-", function(object, value) standardGeneric("randomeffects<-"))

GNUSE <- function(object,medianSE=NULL,type=c("plot","values","stats","density"),...){
  type <- match.arg(type)
  if(length(se.exprs(object))==0) stop("Object does not contain standard errors.")

  if(is.null(medianSE)){
    platform <- gsub("..entrezg", "", annotation(object))
    pkg <- paste(platform, "frmavecs", sep="")
    require(pkg, character.only=TRUE, quietly=TRUE) || stop(paste(pkg, "package must be installed first"))
    vecdataname <- paste(annotation(object), "frmavecs", sep="")
    data(list=eval(vecdataname))
    medianSE <- get(vecdataname)$medianSE
  }

  if(nrow(se.exprs(object))!=length(medianSE)) stop("nrow(se.exprs(object)) does not equal length(medianSE). This may be caused by ExonFeatureSet or GeneFeatureSet objects not summarized at the probeset level.")
  
  gnuses <- sweep(se.exprs(object),1,medianSE,FUN='/')
  if(any(is.na(gnuses))) message("Some GNUSE values are NA. This is often due to probesets with only 1 probe.")
      
  if (type == "values"){
    return(gnuses)
  } else if (type == "density"){
    plotDensity(gnuses, ...)
  } else if (type=="stats"){
    Medians <- apply(gnuses,2,median, na.rm=TRUE)
    Quantiles <- apply(gnuses,2,quantile,prob=c(0.25,0.75,0.95,0.99), na.rm=TRUE)
    nuse.stats <- rbind(Medians,Quantiles[2,] - Quantiles[1,],Quantiles[3,],Quantiles[4,])
    rownames(nuse.stats) <- c("median","IQR","95%","99%")
    return(nuse.stats)
  }
  if (type == "plot"){
    boxplot(data.frame(gnuses), ...)
  }
}

barcode <- function(object, platform=NULL, mu=NULL, tau=NULL, cutoff=6.5, output="binary"){

  if(!class(object) %in% c("matrix", "ExpressionSet", "frmaExpressionSet")
     & !is.vector(object)){
      stop("Object must be one of: vector, matrix, ExpressionSet, or frmaExpressionSet.")
  }

  if(!output %in% c("binary", "z-score", "p-value", "lod")){
      stop("Output must be one of: binary, z-score, p-value, lod.")
  }

  if(is.vector(object)) object <- as.matrix(object)
  
  if(is.matrix(object)){
    if(is.null(platform) & (is.null(mu) | is.null(tau))){
        stop("If object is of class matrix or vector, platform cannot be NULL if mu or tau is NULL.")
    }
  }

  if(class(object) %in% c("ExpressionSet", "frmaExpressionSet")){
    if(is.null(platform)){
      if(annotation(object) %in% c("hgu133a", "hgu133a2", "hgu133plus2", "mouse4302", "hugene.1.0.st.v1", "mogene.1.0.st.v1")){
        if(cleancdfname(annotation(object)) == "hgu133acdf") platform <- "GPL96"
        if(cleancdfname(annotation(object)) == "hgu133a2cdf") platform <- "GPL571"
        if(cleancdfname(annotation(object)) == "hgu133plus2cdf") platform <- "GPL570"
        if(cleancdfname(annotation(object)) == "mouse4302cdf") platform <- "GPL1261"
        if(annotation(object) == "hugene.1.0.st.v1") platform <- "GPL6244"
        if(annotation(object) == "mogene.1.0.st.v1") platform <- "GPL6246"
      } else stop("Microarray platform given by object annotation not recognized -- please supply platform type.")
    } else {
      if(!platform %in% c("GPL96", "GPL570", "GPL571", "GPL1261", "GPL6244", "GPL6246")){
          stop("Platform must be one of: GPL96, GPL570, GPL571, GPL1261", "GPL6244", "GPL6246")
      }
    }
    object <- as.matrix(exprs(object))
  }

  if(is.null(mu) | is.null(tau)){
    if(!platform %in% c("GPL96", "GPL570", "GPL571", "GPL1261", "GPL6244", "GPL6246")){
        stop("mu and tau must be non-NULL unless platform is one of: GPL96, GPL570, GPL571, GPL1261, GPL6244, GPL6246")
    }
    if(platform=="GPL96") pkg <- "hgu133afrmavecs"
    if(platform=="GPL570") pkg <- "hgu133plus2frmavecs"
    if(platform=="GPL571") pkg <- "hgu133a2frmavecs"
    if(platform=="GPL1261") pkg <- "mouse4302frmavecs"
    if(platform=="GPL6244") pkg <- "hugene.1.0.st.v1frmavecs"
    if(platform=="GPL6246") pkg <- "mogene.1.0.st.v1frmavecs"
    require(pkg, character.only=TRUE, quietly=TRUE) || stop(paste(pkg, "package must be installed first"))
    data(list=gsub("frma", "barcode", pkg))
    bcparams <- get(gsub("frma", "barcode", pkg))
    mu <- bcparams$mu
    tau <- bcparams$tau
    names(mu) <- names(tau) <- names(bcparams$entropy)
  } else{
    if(length(mu)!=length(tau)) stop("Lengths of mu and tau must be equal.")
  }

  if(is.null(rownames(object))){
    warning("Object does not contain rownames; therefore, matching between object and barcode parameters is not guarenteed.")
    if(nrow(object)!=length(mu)) stop("Number of rows of object does not equal length of barcode parameters (mu and tau).")
  }

  if(is.null(names(mu)) | is.null(names(tau))){
    warning("Either mu or tau does not contain names; therefore, matching between object and barcode parameters is not guarenteed.")
  } else{
    if(!identical(names(mu),names(tau))) stop("Names of mu and tau must be identical.")
  }

  if(nrow(object)!=length(mu)){
      if(platform %in% c("GPL6244", "GPL6246")) message("The following error can likely be fixed by setting target=\"core\" in the frma function.")
      stop("Number of rows / features of object does not equal length of barcode parameters (mu and tau).")
  }

  if((!is.null(rownames(object))) & (!is.null(names(mu))) & (!is.null(names(tau)))){
      map <- match(rownames(object),names(mu))
      mu <- mu[map]
      tau <- tau[map]
      if(!identical(rownames(object), names(mu))){
          stop("Mismatch between rownames of data object and names of mu and unable to create unique mapping.")
      }
  }
  
  if(output %in% c("p-value", "lod", "binary")){
    pval <- pnorm(object, mean=mu, sd=tau, lower.tail=FALSE)
    if(output == "p-value"){
      colnames(pval) <- colnames(object)
      rownames(pval) <- rownames(object)
      return(pval)
    } else{
      lod <- -log10(pval)
      if(output == "lod"){
        colnames(lod) <- colnames(object)
        rownames(lod) <- rownames(object)
        return(lod)
      } else{
        bc <- matrix(as.integer(lod > cutoff), ncol=ncol(object))
        colnames(bc) <- colnames(object)
        rownames(bc) <- rownames(object)
        return(bc)
      }
    }
  }

  if(output == "z-score"){
    ##z <- sweep(sweep(e, 1, mu), 1, tau, FUN="/")
    z <- (object-mu)/tau
    colnames(z) <- colnames(object)
    rownames(z) <- rownames(object)
    return(z)
  }
}

frma <- function(object, background="rma", normalize="quantile", summarize="robust_weighted_average", target="probeset", 
                 input.vecs=NULL, output.param=NULL, verbose=FALSE){

  ## check input
  if(!class(object) %in% c("AffyBatch", "ExonFeatureSet", "GeneFeatureSet")) stop("object must be of class AffyBatch, ExonFeatureSet, or GeneFeatureSet.")

  if(class(object)=="ExonFeatureSet") target <- match.arg(target, c("core", "full", "extended", "probeset"))
  if(class(object)=="GeneFeatureSet") target <- match.arg(target, c("core", "probeset"))
  if(verbose & class(object)=="ExonFeatureSet") message(paste("Exon summarization at ", target, " level.\n", sep=""))
  if(verbose & class(object)=="GeneFeatureSet") message(paste("Gene summarization at ", target, " level.\n", sep=""))

  if(!background %in% c("none", "rma")) stop("background must be either none or rma")
  if(!normalize %in% c("none", "quantile")) stop("normalize must be either none or quantile")
  if(!summarize %in% c("median_polish", "average", "median", "weighted_average", "robust_weighted_average", "random_effect")){
    stop("summarize must be one of: median_polish, average, median, weighted_average, robust_weighted_average, random_effect")
  }

  if(summarize=="random_effect" & class(object)=="ExonFeatureSet") stop("Summarization method random_effect is not implemented for ExonFeatureSet objects.")
  if(summarize=="random_effect" & class(object)=="GeneFeatureSet") stop("Summarization method random_effect is not implemented for GeneFeatureSet objects.")
  
  ## determine data platform
  if(class(object)=="AffyBatch"){
    cdfname <- cleancdfname(cdfName(object))
    tmp <- gsub("cdf","",cdfname)
    platform <- gsub("..entrezg", "", tmp)
    vecdataname <- paste(tmp, "frmavecs", sep="")
  }
  if(class(object)%in%c("ExonFeatureSet","GeneFeatureSet")){
    cdfname <- annotation(object)
    platform <- gsub("pd.", "", cdfname, fixed=TRUE)
    vecdataname <- paste(platform, "frmavecs", sep="")
  }
  
  ## load frmavecs unless supplied as input 
  if(is.null(input.vecs)){
    pkg <- paste(platform, "frmavecs", sep="")
    require(pkg, character.only=TRUE, quietly=TRUE) || stop(paste(pkg, "package must be installed first"))
    data(list=eval(vecdataname))
    input.vecs <- get(vecdataname)
  } else {
    if(summarize=="random_effect" & (any(input.vecs$probeVarBetween==0) | any(input.vecs$probeVarWithin==0))){
      stop("If summarize method is random_effect then probeVarBetween and probeVarWithin must be greater than zero for all probes.")
    }    
  }
  
  ## call the appropriate frma worker function based on the object class
  if(class(object)=="AffyBatch"){
    cdfname <- cleancdfname(cdfName(object))
    platform <- gsub("cdf","",cdfname)
    output <- frmaAffyBatch(object, background, normalize, summarize, input.vecs, output.param, verbose) 
  }
  if(class(object)%in%c("ExonFeatureSet","GeneFeatureSet")){
    cdfname <- annotation(object)
    platform <- gsub("pd.","",cdfname,fixed=TRUE)
    output <- frmaFeatureSet(object, background, normalize, summarize, target, input.vecs, output.param, verbose)
  }

  ## create a new ExpressionSet or frmaExpressionSet to hold the output
  if(is.null(output.param)){
    if(is.null(output$stderr)){
      e <- new("ExpressionSet", assayData=assayDataNew(exprs=output$exprs), annotation=platform, phenoData=phenoData(object), experimentData=experimentData(object))
    } else{
      e <- new("ExpressionSet", assayData=assayDataNew(exprs=output$exprs, se.exprs=output$stderr), annotation=platform, phenoData=phenoData(object), experimentData=experimentData(object))
    }
  } else {
    if(is.null(output$stderr)){
      e <- new("frmaExpressionSet", assayData=assayDataNew(exprs=output$exprs), annotation=platform, phenoData=phenoData(object), experimentData=experimentData(object))
    } else{
      e <- new("frmaExpressionSet", assayData=assayDataNew(exprs=output$exprs, se.exprs=output$stderr), annotation=platform, phenoData=phenoData(object), experimentData=experimentData(object))
    }
    if("weights" %in% output.param) w <- output$weights else w <- matrix(nrow=0, ncol=0)
    if("residuals" %in% output.param) r <- output$residuals else r <- matrix(nrow=0, ncol=0)
    if("random_effects" %in% output.param) g <- output$gammas else g <- matrix(nrow=0, ncol=0)
    weights(e) <- w
    residuals(e) <- r
    randomeffects(e) <- g
  }

  return(e)
}

% automatic manuscript creation for frma
% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{frma: Preprocessing for single arrays and array batches}
%\VignetteDepends{frma, hgu133afrmavecs, frmaExampleData}
%\VignettePackage{frma}
\documentclass[12pt]{article}
\usepackage{hyperref}
\usepackage[authoryear, round]{natbib}

\textwidth=6.2in
\textheight=8.5in
\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\R}{{$\mathbf{R}$}}
\newcommand\Rpackage[1]{{\textsf{#1}\index{#1 (package)}}}
\newcommand\dataset[1]{{\textit{#1}\index{#1 (data set)}}}
\newcommand\Rclass[1]{{\textit{#1}\index{#1 (class)}}}
\newcommand\Rfunction[1]{{{\small\texttt{#1}}\index{#1 (function)}}}
\newcommand\Rfunarg[1]{{\small\texttt{#1}}}
\newcommand\Robject[1]{{\small\texttt{#1}}}

\author{Matthew N. McCall}

\begin{document}
\title{Frozen Robust Multi-Array Analysis and the Gene Expression Barcode}
\maketitle
\tableofcontents

The \Rpackage{frma} implements methods for the preprocessing (frma)
and analysis (barcode) of single microarrays. The fRMA algorithm
allows the user to preprocess arrays individually while retaining the
advantages of multi-array preprocessing methods such as RMA. The Gene
Expression Barcode provides estimates of absolute expression rather
than relative expression.

\section{Frozen Robust Multiarray Analysis (fRMA)}
Frozen RMA (fRMA) is a microarray preprocessing algorithm that allows
one to analyze microarrays individually or in small batches and then
combine the data for analysis. This is accomplished by utilizing
information from the large publicly available microarray
databases. Specifically, estimates of probe-specific effects and
variances are precomputed and frozen. Then, with new data sets, these
frozen parameters are used in concert with information from the new
array(s) to preprocess the data.

The fRMA is particularly useful when it is not feasible to preprocess
all of the data simultaneously. Such situations often arise when:
\begin{itemize}
  \item large meta-analyses require one to preprocess more data than
    can be stored in memory,  
  \item datasets grow incrementally and it would be laborious to
    preprocess all of the data each time a new array is added,
  \item microarrays are used to aid in clinical diagnosis and treatment, one
    needs to obtain information based on a single sample hybridized to
    a single microarray.
\end{itemize}    
Additionally, fRMA down-weights probes that appear to be
\emph{batchy}, those that are most susceptible to batch-effects.

\subsection{From CEL files to expression estimates}
For Affymetrix microarray data, it is customary to view CEL files as
the starting point for preprocessing and analysis. One or more CEL
files can be read into \R{} using the \Rfunction{ReadAffy} function
from the \Rpackage{affy} package to produce an \Robject{AffyBatch}
object or using the \Rfunction{read.celfiles} function from the
\Rpackage{oligo} to produce a \Robject{FeatureSet}. 

The goal of fRMA is to obtain reliable gene-level intensities from the
raw microarray data. This amounts to converting raw probe-level
intensities into background-corrected, normalized, and summarized
gene-level intensities. The \Rfunction{frma} function takes an
\Robject{AffyBatch} or \Robject{FeatureSet} as input and produces an
object containing gene-level expression values. This object can take
one of two forms, an \Robject{ExpressionSet} or a
\Robject{frmaExpressionSet}, depending upon the additional information
that is requested. The gene-level intensities stored in both these
objects can be accessed using the \Rfunction{exprs} method.  

In addition to the raw data, the fRMA algorithm requires a number of
\emph{frozen} parameter vectors. Among these are the reference
distribution to which the data are normalized and the probe-effect
estimates. We have computed these frozen parameters for many popular
Affymetrix platforms. The data for each of these platforms is stored
in an \R{} package of the form \Rpackage{$<$platform$>$frmavecs}. By
default, the \Rfunction{frma} function attempts to load the
appropriate data package for the input data object.

The remainder of this section describes typical use of the
\Rfunction{frma}. For details of the statistical methodology
implemented in the frma function, please read the following papers:

McCall MN, Bolstad BM, and Irizarry RA (2010). Frozen Robust
Multi-Array Analysis (fRMA), Biostatistics, 11(2):242-253.

McCall MN, Jaffee HA, Irizarry RA (2012). fRMA ST: Frozen robust
multiarray analysis for Affymetrix Exon and Gene ST arrays,
Bioinformatics, 28(23):3153-3154.

The following is a simple description of how to preprocess a single CEL file using the default version of frma:
\begin{enumerate}
\item Download and install frma and the appropriate frozen parameter package (i.e. hgu133afrmavecs, hgu133plus2frmavecs, etc.).
\item Load the frma package.
<<echo=T,results=hide>>=
library(frma)
@ 
\item Read in the data as either an \Robject{AffyBatch} or \Robject{FeatureSet}
  object. For the early Affymetrix platforms (e.g. HGU133plus2), you
  should use the \Rfunction{ReadAffy} function from the
  \Rpackage{affy} package to read in the CEL files. For the later
  Affymetrix platforms (e.g. Gene ST or Exon ST), you should use the
  \Rfunction{read.celfiles} function from the \Rpackage{oligo} package.
\item In this vignette we will load an example \Robject{AffyBatch} object:  
<<echo=T,results=hide>>=
library(frmaExampleData)
data(AffyBatchExample)
@ 
\item Preprocess the raw data object using the default version of frma.
<<echo=T,results=hide>>=
object <- frma(AffyBatchExample)
@ 
\end{enumerate}

The final \Robject{object} will be an \Robject{ExpressionSet} or
\Robject{frmaExpressionSet}. The latter is an extension of the
ExpressionSet class to hold additional information related to the
preprocessing procedure such as weights and residuals. To obtain a
matrix of gene-level expression values, enter the following command:
<<echo=T,results=hide>>=
e <- exprs(object)
@ 

To assess the quality of the expression estimates, one can use the \Rfunction{GNUSE} function:
<<echo=T,results=verbatim>>=
GNUSE(object,type="stats")
@ 
The GNUSE is a modification of the NUSE quality metric that allows
one to assess the quality of individual samples relative to the large
training data set used to compute the frozen parameter vectors. The
GNUSE is described in more detail in the following paper:

McCall MN, Murakami PN, Lukk M, Huber W, Irizarry RA (2011). Assessing
Affymetrix GeneChip Microarray Quality, BMC Bioinformatics, 12:137.

Depending on the number of CEL files and on the memory available to
your system, you might experience errors like `Cannot allocate vector
\ldots'.  An obvious option is to increase the memory available to
your R process (by adding memory and/or closing external
applications). You might also consider analyzing the data in smaller
batches. Recall that fRMA allows you to preprocess data separately and
then combine the preprocessed data for further analysis.

\subsection{Advanced Options}
The default arguments to the \Rfunction{frma} function will be
sufficient for most users; however, additional options have been
implemented to allow the user to control each stage of the
preprocessing, as well as, the information returned by the
\Rfunction{frma} function. This flexibility is instrumental in
allowing users to easily explore alternative methods of preprocessing.

\subsubsection{Summarization Methods}
\emph{Summarization} refers to the method used to combine probe-level
expression values to obtain gene-level expression estimates. There are
several summarization methods that one can choose from when running
frma. A brief description of each of the methods follow:
\begin{itemize}
  \item {\bf{average}:} subtract the probe-effect and then compute the mean of the probes in each probeset.
  \item {\bf{median}:} subtract the probe-effect and compute the median of the probes in each probeset.
  \item {\bf{median\_polish}:} this is the same as the median summarization because the probe-effects have already been removed.
  \item {\bf{weighted average}:} compute a weighted average of the probes in each probeset with weights equal to the inverse of the sum of the precomputed within and between batch variance estimates.
  \item {\bf{robust\_weighted\_ average}:} compute a weighted average of the probes in each probeset with weights equal to the weights returned by an M-estimation procedure divided by the sum of the precomputed within and between batch variance estimates. 
  \item {\bf{random\_ effect}:} the robust weighted average method adapted for a batch of new arrays (see the fRMA paper for details).
\end{itemize}

\subsubsection{Input Vectors}
While the vast majority of users will use the precomputed vectors
provided in the \Rpackage{frmavecs} packages, the \Rfunction{frma}
function will accept user-supplied frozen parameter vectors. The
\Rpackage{frmaTools} package contains functions to create your own
frozen parameter vectors. There are several situations in which
creating your own frozen parameter vectors may be beneficial. These
are described in detail in:

McCall MN and Irizarry RA (2011). Thawing Frozen Robust Multi-array
Analysis (fRMA), BMC Bioinformatics, 12:369.

If you create your own frozen parameter vectors using functions in the
\Rpackage{frmaTools} package, the vectors will already be in the
correct format: a list with elements normVec, probeVec,
probeVarBetween, probeVarWithin, probesetSD, and medianSE. A
description of each of these elements follows:
\begin{itemize}
  \item {\bf{normVec}:} a vector containing values of the reference distribution to which samples will be quantile normalized
  \item {\bf{probeVec}:} a vector of probe-effect estimates
  \item {\bf{probeVarBetween}:} a vector of the between batch variance for each probe
  \item {\bf{probeVarWithin}:} a vector of the within batch variance for each probe
  \item {\bf{probesetSD}:} a vector of average within probeset standard deviations
  \item {\bf{medianSE}:} a vector of median standard errors of the expression estimates (used by the GNUSE function)
\end{itemize}

\subsubsection{Output Parameters}
While the default is to only return the gene-level expression
estimates and if applicable their standard errors, the frma function
can also return additional information about the estimates depending
on the summarization method chosen. A description of the arguments
that can be included in the output.param argument follows:
\begin{itemize}
  \item {\bf{weights}:} the weights from the M-estimation procedure
  \item {\bf{residuals}:} the residuals from fitting the probe-level model
  \item {\bf{randomeffects}:} estimated random effects from fitting the probe-level model with random effect summarization
\end{itemize}

Note that not all of these outputs are available for all of the summarization
methods.

\section{Creation of Gene Expression Barcodes}
The barcode algorithm is designed to estimate which genes are
expressed and which are unexpressed in a given microarray
hybridization. This is accomplished by: (1) using the distribution of
observed $log_2$ intensities across a wide variety of tissues to
estimate an expressed and an unexpressed distribution for each gene,
and (2) using these estimated distributions to determine which genes
are expressed / unexpressed in a given sample. The first step is
accomplished by fitting a hierarchical mixture model to the plethora
of publicly available data. The second step is accomplished by
determining where the observed intensities from the new array fall in
the estimated distributions. The default output of the
\Rfunction{barcode} function is a vector of ones and zeros denoting
which genes are estimated to be expressed (ones) and unexpressed
(zeros). We call this a \emph{gene expression barcode}.

For more details about the Gene Expression Barcode, please read the following papers:

McCall MN, Uppal K, Jaffee HA, Zilliox MJ, and Irizarry RA (2011). The
Gene Expression Barcode: leveraging public data repositories to begin
cataloging the human and murine transcriptomes, Nucleic Acids
Research, 39:D1011-D1015.

McCall MN, Jaffee HA, Zelisko SJ, Sinha N, Hooiveld G, Irizarry RA,
Zilliox MJ (2014). The Gene Expression Barcode 3.0: improved data
processing and mining tools, Nucleic Acids Research, 42(D1):D938-D943.

\subsection{Getting Started}
To create a gene expression barcode, one needs estimates of the gene
expression distributions -- specifically the mean and variance of the
unexpressed distribution for each gene. We have computed these for
several popular Affymetrix platforms. To use one of these, simply
preprocess your data using the default options of \Rfunction{frma} and
then run the \Rfunction{barcode} function on the resulting object.

Similar to the \Rfunction{frma} function, the \Rfunction{barcode}
function requires platform specific precomputed parameters. These
parameters are stored in the same \R{} packages as the frozen
parameter vectors used by \Rfunction{frma}. In the default
implementation, the \Rfunction{barcode} function attempts to load the
appropriate set of parameters for the given \Robject{ExpressionSet} or
\Robject{frmaExpressionSet} object. It is also possible for the user
to supply the necessary parameters via optional arguments.

\subsubsection{Example}
\begin{enumerate}
\item Download and install the frma package and the appropriate data
package(s) (i.e. hgu133afrmavecs).
\item Load the frma package.
<<echo=T,results=hide>>=
library(frma)
@ 
\item Read in the data and preprocess using the default options.
<<echo=T,results=hide>>=
library(frmaExampleData)
data(AffyBatchExample)
object <- frma(AffyBatchExample)
@ 
\item Convert the expression values to a gene expression barcode.
<<echo=T,results=hide>>=
bc <- barcode(object)
@ 
\end{enumerate}

\subsection{Output Options}

The default output of the \Rfunction{barcode} function is to return a vector of ones (expressed) and zeros (unexpressed); however, there are alternative output options. A brief description of each of these follows:
\begin{itemize}
  \item {\bf{weight}:} a vector of weights which roughly correspond to the probability of expression for each gene.
  \item {\bf{z-score}:} a vector of z-scores under the unexpressed normal distribution for each gene.
  \item {\bf{p-value}:} a vector of p-values under the unexpressed normal distributionfor each gene.
  \item {\bf{lod}:} a vector of LOD scores under the unexpressed normal distributionfor each gene.
\end{itemize}

\end{document}

\name{GNUSE}
\alias{GNUSE}
\title{GNUSE}
\description{
  Computes the generalized normalized unscaled standard error (a measure
  of microarray quality).
}
\usage{
  GNUSE(object, medianSE=NULL, type=c("plot", "values", "stats", "density"), ...)
}
\arguments{
  \item{object}{an ExpressionSet or frmaExpressionSet containing
    standard errors produced by frma}
  \item{medianSE}{median standard errors to be used. If NULL, these are
    obtained from the correct frmavecs package.}
  \item{type}{the desired output}
  \item{...}{additional graphical parameters for types plot or density}
}
\value{
  If type is plot, boxplots of GNUSE values are displayed. If type is
  values, the GNUSE values are returned. If type is stats, the median,
  IQR, 95th, and 99th percentiles are reported. If type is density, a
  density plots of GNUSE values are displayed.
}
\author{Matthew N. McCall}
\examples{
  library(frma)
  library(frmaExampleData)
  data(AffyBatchExample)
  object <- frma(AffyBatchExample)
  GNUSE(object, type="stats")
}
\keyword{manip}

\name{frmaExpressionSet-class}
\docType{class}
\alias{class:frmaExpressionSet}
\alias{frmaExpressionSet-class}
\alias{frmaExpressionSet}
\alias{frmaExpressionSet-methods}
\alias{initialize,frmaExpressionSet-method}
\alias{se.exprs,frmaExpressionSet-method}
\alias{se.exprs,ExpressionSet-method}
\alias{weights,frmaExpressionSet-method}
\alias{residuals,frmaExpressionSet-method}
\alias{randomeffects,frmaExpressionSet-method}
\alias{as.ExpressionSet,frmaExpressionSet-method}
\alias{se.exprs,frmaExpressionSet-method}
\alias{se.exprs<-}
\alias{se.exprs<-,frmaExpressionSet-method}
\alias{se.exprs<-,ExpressionSet-method}
\alias{weights,frmaExpressionSet-method}
\alias{weights<-}
\alias{weights<-,frmaExpressionSet-method}
\alias{residuals,frmaExpressionSet-method}
\alias{residuals<-}
\alias{residuals<-,frmaExpressionSet-method}
\alias{randomeffects}
\alias{randomeffects,frmaExpressionSet-method}
\alias{randomeffects<-}
\alias{randomeffects<-,frmaExpressionSet-method}
\title{
  Class to Contain and Describe High-Throughput Expression Level Assays
  preprocessed with fRMA}
\description{This is a class representation for fRMA-preprocessed expression
  data. \code{frmaExpressionSet} class is derived from
  \code{ExpressionSet}, and requires a matrix named \code{exprs}
  and optionally matrices named \code{se.exprs}, \code{weights}, and \code{residuals}.}
\section{Extends}{
  Extends class \code{ExpressionSet}.
  }
\section{Creating Objects}{

  \code{new("frmaExpressionSet",
    exprs = new("matrix"),
    se.exprs = new("matrix"),
    weights=new("matrix"),
    residuals=new("matrix"),
    randomeffects=new("matrix"),
    phenoData = new("AnnotatedDataFrame"),
    featureData = new("AnnotatedDataFrame"),
    experimentData = new("MIAME"),
    annotation = new("character"),
    ...)
  }  

  This creates a \code{frmaExpressionSet} with \code{assayData}
  implicitly created to contain \code{exprs} and \code{se.exprs}. The
  only required named arguments is \code{exprs}. Three optional named
  matrices, \code{weights}, \code{residuals}, and \code{randomeffects}
  can be added to the object.

  \code{new("frmaExpressionSet",
    assayData = assayDataNew(exprs=new("matrix"),se.exprs=new("matrix")),  
    weights=new("matrix"),
    residuals=new("matrix"),
    randomeffects=new("matrix"),
    phenoData = new("AnnotatedDataFrame"),
    featureData = new("AnnotatedDataFrame"),
    experimentData = new("MIAME"),
    annotation = new("character"),
    ...)
  }

  This creates a \code{frmaExpressionSet} with \code{assayData} provided
  explicitly. In this form, the only required named argument is
  \code{assayData}. Three optional named matrices, \code{weights},
  \code{residuals}, and \code{randomeffects} can be added to the object.
  }
    
\section{Slots}{
  \describe{
    \item{\code{se.exprs}:}{standard errors for the expression estimates}
    \item{\code{weights}:}{weights used in the summarization step}
    \item{\code{residuals}:}{residuals from fitting the probe-level
      model}
    \item{\code{randomeffects}:}{random effect estimates from fitting
  the probe-level model using random effect summarization}
  }
  Inherited from \code{ExpressionSet}:
  \describe{
    \item{\code{assayData}:}{Contains matrices with equal dimensions,
      and with column number equal to
      \code{nrow(phenoData)}. \code{assayData} must contain a matrix
      \code{exprs} with rows representing features and columns
      representing samples. It may also contain a matrix \code{se.exprs}
    containing standard errors.}  
    \item{\code{phenoData}:}{See \code{eSet}}
    \item{\code{annotation}:}{See \code{eSet}}
    \item{\code{featureData}:}{See \code{eSet}}
    \item{\code{experimentData}:}{See \code{eSet}} 
  }
}

\section{Methods}{
  Class-specific methods:
  \describe{
    \item{\code{se.exprs(frmaExpressionSet)}}{Access elements named
      \code{se.exprs} in the \code{AssayData-class} slot.}
    \item{\code{weights(frmaExpressionSet)}}{Access elements named
      \code{weights}}
    \item{\code{residuals(frmaExpressionSet)}}{Access elements named
      \code{residuals}}
    \item{\code{randomeffects(frmaExpressionSet)}}{Access elements named
      \code{randomeffects}}
  }

  For derived methods (see \code{ExpressionSet}).
}

\seealso{
  \code{eSet-class}, \code{ExpressionSet-class}, \code{frma}. 
} 

\examples{
# create an instance of frmaExpressionSet
new("frmaExpressionSet")
}

\keyword{classes}

\name{barcode}
\alias{barcode}
\title{Gene Expression Barcode}
\description{
  This function converts expression values produced via fRMA to a gene
  expression barcode.
}
\usage{
  barcode(object, platform=NULL, mu=NULL, tau=NULL, cutoff=6.5, output="binary")
}
\arguments{
  \item{object}{a vector or matrix of expression values or an
    ExpressionSet or frmaExpressionSet produced by frma}
  \item{platform}{the platform of the input data. One of GPL96, GPL570,
    GPL571, GPL1261, GPL6244, GPL6246. Required if object is a vector or
    matrix and either mu or tau is NULL.}
  \item{mu}{the mean of the unexpressed distribution. If NULL then
    precomputed values are used if possible.}
  \item{tau}{the standard deviation of the unexpressed distribution. If
    NULL then precomputed values are used if possible.}
  \item{cutoff}{the lod score cutoff used if output is binary.}
  \item{output}{the desired values to be returned. Options are: p-value,
    z-score, lod, or binary.}
}
\value{
  A matrix containing the type of output specified by the output
  parameter. The option \emph{binary} creates a gene expression barcode
  where 1s denote expressed genes and 0s denote unexpressed genes. The
  option \emph{p-value} returns the p-values for the expression values
  under the unexpressed distribution. The option \emph{lod} returns the
  LOD scores for expression values under the unexpressed
  distribution. The option \emph{z-score} returns the z-scores for the
  expression values under the unexpressed distribution. 
}
\author{Matthew N. McCall}
\examples{
  library(frma)
  library(frmaExampleData)
  data(AffyBatchExample)
  object <- frma(AffyBatchExample)
  bc <- barcode(object)
}
\keyword{manip}

\name{frma}
\alias{frma}
\title{Frozen Robust Multi-Array Analysis}
\description{
  This function preprocesses an AffyBatch, ExonFeatureSet, or GeneFeatureSet object using the fRMA method.
}
\usage{
frma(object, background="rma", normalize="quantile",
    summarize="robust_weighted_average", target="probeset",
    input.vecs=NULL, output.param=NULL, verbose=FALSE) 
}
\arguments{
  \item{object}{an AffyBatch, ExonFeatureSet, or GeneFeatureSet}
  \item{background}{type of background correction to perform: either
    "none" or "rma".}
  \item{normalize}{type of normalization to perform: either "none" or
    "quantile".}  
  \item{summarize}{type of summarization to perform: one of
    "median\_polish", "average", "median", "weighted\_average",
    "robust\_weighted\_average", "random\_effect".}
  \item{target}{summarization level for exon and gene arrays. Must be one of:
    probeset, core, full (exon only), extended (exon only).}
  \item{input.vecs}{a list of vectors to be used in preprocessing. If NULL, 
    the correct package with pre-made vectors is loaded if it has been installed. 
    These packages are of the form: <platform>frmavecs.}
  \item{output.param}{a vector of output elements to return. By default
    only the expression values and standard errors (if applicable) are
    returned. Additional options are: "residuals", "weights", "random_effects"}
  \item{verbose}{logical value. If TRUE then some messages are displayed
    while the function runs.}
}
\value{
  The function returns an ExpressionSet if output.param=NULL or an
  frmaExpressionSet otherwise.
}
\author{Matthew N. McCall}
\examples{
  library(frmaExampleData)
  data(AffyBatchExample)
  object <- frma(AffyBatchExample)
}
\keyword{manip}
